import objectfile

NEGATIVE = -1

class LMCException(Exception): pass
class Halt(LMCException): pass
class BadInstruction(LMCException): pass
class Overflow(LMCException): pass
class PCOverflow(Overflow): pass

class Register(object):
	def __init__(self, data):
		assert 0 <= data <= 999 or data == NEGATIVE
		self.data = data

	@property
	def instruction(self):
		if self.data == NEGATIVE:
			return None
		return self.data / 100

	@property
	def argument(self):
		if self.data == NEGATIVE:
			return None
		return self.data % 100

	@property
	def instruction_mnemonic(self):
		i, a = self.instruction, str(self.argument).zfill(2)
		if self.data == 000:
			return ("HLT", '')
		elif self.instruction is None:
			return ("NEG", '')
		elif self.instruction == 1:
			return ("ADD", a)
		elif self.instruction == 2:
			return ("SUB", a)
		elif self.instruction == 3:
			return ("STA", a)
		elif self.instruction == 5:
			return ("LDA", a)
		elif self.instruction == 6:
			return ("BRA", a)
		elif self.instruction == 7:
			return ("BRZ", a)
		elif self.instruction == 8:
			return ("BRP", a)
		elif self.data == 901:
			return ("INP", '')
		elif self.data == 902:
			return ("OUT", '')
		else:
			return ("DATA", '')

	def add(self, v):
		assert 0 <= v <= 999
		self.data += v
		if self.data > 999:
			raise Overflow
	
	def substract(self, v):
		assert 0 <= v <= 999
		self.data -= v
		if self.data < 0:
			self.data = -1

	def __str__(self):
		im, am = self.instruction_mnemonic
		return "%3s %3s%3s" % (str(self.data).zfill(3)
					if self.data >= 0 else '', im, am)

class Computer(object):
	def __init__(self, mem=None, acc=None, pc=None, inp=None, out=None,
			on_inp=None, on_out=None):
		if mem is None: mem = [0 for i in xrange(100)]
		if acc is None: acc = Register(0)
		if inp is None: inp = Register(0)
		if out is None: out = Register(0)
		if on_inp is None: on_inp = self.default_on_inp
		if on_out is None: on_out = self.default_on_out
		if pc is None:
			pc = 0
		self.mem = [Register(v) for v in mem]
		self.acc = acc
		self.inp = inp
		self.out = out
		self.on_out = on_out
		self.on_inp = on_inp
		self.pc = pc
	
	def default_on_inp(self):
		return int(raw_input("input> "))

	def default_on_out(self, v):
		print "Output: ", str(self.out)
	
	def _step_pc(self):
		self.pc += 1
		if self.pc > 99:
			raise PCOverflow
	
	def _jump_pc(self, a):
		assert 0 <= a <= 99
		self.pc = a
	
	def run(self):
		try:
			while True:
				print self
				self.step()
		except Halt:
			pass

	def step(self):
		reg = self.mem[self.pc]
		i, a = reg.instruction, reg.argument
		if reg.data == 000:		# HLT
			raise Halt
		elif i is None:
			raise BadInstruction
		elif i == 1:	# ADD
			self.acc.add(self.mem[a].data)
			self._step_pc()
		elif i == 2:	# SUB
			self.acc.substract(self.mem[a].data)
			self._step_pc()
		elif i == 3:	# STA
			self.mem[a].data = self.acc.data
			self._step_pc()
		elif i == 5:	# LDA
			self.acc.data = self.mem[a].data
			self._step_pc()
		elif i == 6:	# BRA
			self._jump_pc(a)
		elif i == 7:	# BRZ
			if self.acc.data == 0:
				self._jump_pc(a)
			else:
				self._step_pc()
		elif i == 8:	# BRP
			if self.acc.data == NEGATIVE:
				self._step_pc()
			else:
				self._jump_pc(a)
		elif reg.data == 901:		# INP
			self.inp.data = self.on_inp()
			self.acc.data = self.inp.data
			self._step_pc()
		elif reg.data == 902:		# OUT
			self.out.data = self.acc.data
			self.on_out(self.out.data)
			self._step_pc()
		else:
			raise BadInstruction
	
	def __str__(self):
		ret = ''
		for i in xrange(20):
			for j in xrange(5):
				flag = ''
				if i+20*j == self.pc:
					flag = '*'
				ret += "%1s%-15s" % (flag, self.mem[i+20*j])
			ret += '\n'
		ret += '             acc %-11s inp %-11s out %-15s' % (
				self.acc, self.inp, self.out)
		return ret

def main():
	import optparse
	usage = 'usage: %prog [options] objectfile'
	parser = optparse.OptionParser(usage=usage)
	(options, args) = parser.parse_args()
	if len(args) == 0:
		parser.print_help()
		return
	with open(args[0], 'r') as f:
		mem = objectfile.load(f)
	c = Computer(mem=mem)
	c.run()

if __name__ == '__main__':
	main()



import objectfile

class LMCAssemblerException(Exception): pass
class SyntaxError(LMCAssemblerException): pass
class LabelNotFound(LMCAssemblerException): pass
class DuplicateLabel(LMCAssemblerException): pass
class UnknownInstruction(LMCAssemblerException): pass

class LMCAssembler(object):
	def __init__(self):
		self.instr = []
	def parse_line(self, l):
		l = l[:-1].strip()
		l = l.split('#')[0]
		if not l:
			return None
		bits = [bit.strip().lower() for bit in l.split() if bit.strip()]
		if len(bits) == 0:
			return None
		if len(bits) > 3:
			raise SyntaxError
		if len(bits) == 1:
			return (None, self.mnemonic_to_code(bits[0]), None)
		elif len(bits) == 2:
			try:
				return (bits[0], self.mnemonic_to_code(bits[1]),
						None)
			except UnknownInstruction:
				return (None, self.mnemonic_to_code(bits[0]),
						bits[1])
		else: # len(bits) == 3
			return (bits[0], self.mnemonic_to_code(bits[1]),
					bits[2])

	def mnemonic_to_code(self, v):
		if v == 'hlt':
			return 0
		elif v == 'inp':
			return 901
		elif v == 'out':
			return 902
		elif v == 'add':
			return 100
		elif v == 'sub':
			return 200
		elif v == 'sta':
			return 300
		elif v == 'lda':
			return 500
		elif v == 'bra':
			return 600
		elif v == 'brz':
			return 700
		elif v == 'brp':
			return 800
		elif v == 'dat':
			return 000
		else:
			raise UnknownInstruction

	def parse_file(self, f):
		for l in f.readlines():
			parsed = self.parse_line(l)
			if parsed is None:
				continue
			self.instr.append(parsed)

	def resolve_labels(self):
		lut = dict()
		for n, l in enumerate(self.instr):
			if l[0] is None:
				continue
			if l[0] in lut:
				raise DuplicateLabel
			lut[l[0]] = n
		for n, l in enumerate(self.instr):
			if l[2] is None:
				continue
			if isinstance(l[2], int):
				continue
			if l[2].isdigit():
				self.instr[n] = (l[0], l[1], int(l[2]))
				continue
			if not l[2] in lut:
				raise LabelNotFound, l[2]
			self.instr[n] = (l[0], l[1], lut[l[2]])
	
	def emit(self):
		ret = []
		for l, c, r in self.instr:
			if not r is None:
				c += r
			ret.append(c)
		return ret

	def assemble(self, f):
		self.parse_file(f)
		self.resolve_labels()
		ret = self.emit()
		return ret + ([0] * (100 - len(ret)))

def assemble(f):
	return LMCAssembler().assemble(f)

def main():
	import optparse
	usage = 'usage: %prog [options] filein [fileout]'
	parser = optparse.OptionParser(usage=usage)
	(options, args) = parser.parse_args()
	if len(args) == 0:
		parser.print_help()
		return
	if len(args) == 1:
		bits = args[0].rsplit('.', 1)
		if bits[-1] == 'lmc':
			bits[-1] = 'lmco'
		args.append('.'.join(bits))
	with open(args[0], 'r') as fi, \
	     open(args[1], 'w') as fo:
		objectfile.save(fo, assemble(fi))

if __name__ == '__main__':
	main()




import zlib
import struct

class UnknownFormat(Exception): pass
class FormatError(Exception): pass

def load(f):
	header = f.read(4)
	if header == 'LMC0':
		return load_lmc0(f)
	else:
		raise UnknownFormat

def load_lmc0(f):
	ret = []
	l = struct.unpack('<H', f.read(2))[0]
	s = zlib.decompress(f.read(l))
	if len(s) != 300:
		raise FormatError
	for i in xrange(100):
		ret.append(int(s[3*i:3*i+3]))
	return ret

def save(f, mem):
	save_lmc0(f, mem)

def save_lmc0(f, mem):
	f.write('LMC0')
	mem = mem + ([0] * (100 - len(mem)))
	r = ''
	for v in mem:
		r += str(v).zfill(3)
	r = zlib.compress(r)
	f.write(struct.pack('<H', len(r)))
	f.write(r)



#!/usr/bin/env python

from setuptools import setup, find_packages
from get_git_version import get_git_version

setup(name='pylmc',
      version=get_git_version(),
      description='Assembler and interpreter for the Little Man Computer',
      author='Bas Westerbaan',
      classifiers = [
	      'Development Status :: 3 - Alpha',
	      'Environment :: Console',
	      'Intended Audience :: Developers',
	      'Intended Audience :: Education',
	      'License :: OSI Approved :: GNU Affero General Public License v3',
	      'Natural Language :: English',
	      'Operating System :: POSIX',
	      'Operating System :: Microsoft',
	      'Programming Language :: Python :: 2.7',
	      'Topic :: Software Development :: Interpreters',
	      'Topic :: Software Development :: Libraries',
	      'Topic :: Software Development :: Assemblers',
	      'Topic :: Software Development :: Code Generators',
	      'Topic :: Software Development :: Disassemblers'
      ],
      author_email='bas@westerbaan.name',
      url='http://github.com/bwesterb/pylmc/',
      packages=['lmc'],
      zip_safe=True,
      package_dir={'lmc': 'src'},
      entry_points = {
	      'console_scripts': [
		      'lmca = lmc.assembler:main',
		      'lmc = lmc.interpreter:main',
	      ],
      }
      )
